#!/usr/bin/env node
// Source: https://github.com/evanw/esbuild/issues/1921#issuecomment-1898197331
import { createRequire } from 'node:module';
import url from 'node:url';
globalThis.require = createRequire(import.meta.url);
globalThis.__filename = url.fileURLToPath(import.meta.url);
// @ts-expect-error The `path` module will anyway be imported by the injected script
globalThis.__dirname = path.dirname(__filename);


// src/bin/index.ts
import { parseArgs } from "util";

// src/bin/commands/init.ts
import childProcess from "child_process";
import fs2 from "fs/promises";
import json5 from "json5";
import ora from "ora";
import path2 from "path";
import util from "util";

// src/bin/utils/file.ts
import fs from "fs/promises";
import path from "path";
var exists = async (file) => {
  try {
    await fs.access(path.join(process.cwd(), file));
  } catch (err) {
    if (err.code === "ENOENT") {
      return false;
    } else {
      throw err;
    }
  }
  return true;
};

// src/bin/commands/init.ts
var exec = util.promisify(childProcess.exec);
var init_default = async (positionals2) => {
  const spinner = ora("Initializing project").start();
  const lastPositional = positionals2[positionals2.length - 1];
  const spaceHandle = lastPositional === "init" ? null : lastPositional;
  if (!spaceHandle) {
    spinner.fail("Please provide a space handle like this:\n$ ronin init my-space");
    process.exit(1);
  }
  if (!await exists("package.json")) {
    spinner.fail("No `package.json` found in the current directory. Please run the command in your project.");
    process.exit(1);
  }
  const packageManager = await exists("bun.lockb") ? "bun" : "npm";
  const packageManagerName = packageManager === "bun" ? "Bun" : "npm";
  spinner.text = `Detected ${packageManagerName} \u2014 installing types package`;
  try {
    const gitignoreExists = await exists(".gitignore");
    if (gitignoreExists) {
      const gitignorePath = path2.join(process.cwd(), ".gitignore");
      const gitignoreContents = await fs2.readFile(gitignorePath, "utf-8");
      if (!gitignoreContents.includes(".ronin")) {
        await fs2.appendFile(gitignorePath, "\n.ronin");
      }
    }
    if (packageManager === "bun") {
      await exec(`bun add @ronin/${spaceHandle} --dev`);
    } else {
      await exec(`npm install @ronin/${spaceHandle} --save-dev`);
    }
    const tsConfigPath = path2.join(process.cwd(), "tsconfig.json");
    const tsConfigExists = await exists("tsconfig.json");
    if (!tsConfigExists) return;
    const contents = await fs2.readFile(tsConfigPath, "utf-8");
    const tsConfig = json5.parse(contents);
    if (!tsConfig.compilerOptions.types || !tsConfig.compilerOptions.types.includes(`@ronin/${spaceHandle}`))
      Object.assign(tsConfig.compilerOptions, {
        types: [...tsConfig.compilerOptions.types || [], `@ronin/${spaceHandle}`]
      });
    await fs2.writeFile(tsConfigPath, JSON.stringify(tsConfig, null, 2));
  } catch (err) {
    if (err instanceof Error && err.message.includes("401")) {
      spinner.fail(`You are not a member of the "${spaceHandle}" space or the space doesn't exist.`);
      process.exit(1);
    }
  }
  spinner.succeed("Project initialized");
};

// src/bin/commands/login.ts
import getPort from "get-port";
import http from "http";
import open from "open";
import ora2 from "ora";

// src/bin/utils/session.ts
import toml from "@iarna/toml";
import fs3 from "fs/promises";
import ini from "ini";
import os from "os";
import path3 from "path";
var CACHE_DIR = path3.join(os.homedir(), ".ronin");
var CACHE_DIR_FILE = path3.join(CACHE_DIR, "session.json");
var getSession = async () => {
  try {
    const contents = await fs3.readFile(CACHE_DIR_FILE, {
      encoding: "utf-8"
    });
    const parsedContents = JSON.parse(contents);
    if (parsedContents?.token) return parsedContents;
  } catch (err) {
    if (err.code !== "ENOENT") throw err;
  }
  return null;
};
var readConfigFile = async (filePath, name, parser) => {
  let npmConfigContents = {};
  try {
    const contents = await fs3.readFile(filePath, { encoding: "utf-8" });
    npmConfigContents = parser(contents);
  } catch (err) {
    if (err.code !== "ENOENT") {
      console.error(`Failed to read ${name} config file at ${filePath}`, err);
    }
  }
  return npmConfigContents;
};
var writeConfigFile = async (filePath, contents) => {
  return fs3.writeFile(filePath, contents, { encoding: "utf-8" });
};
var storeSession = async (token) => {
  try {
    await fs3.stat(CACHE_DIR);
  } catch (err) {
    if (err.code === "ENOENT") {
      await fs3.mkdir(CACHE_DIR);
    } else {
      throw err;
    }
  }
  return writeConfigFile(CACHE_DIR_FILE, JSON.stringify({ token }, null, 2));
};
var storeTokenForNPM = async (token) => {
  const npmConfigFile = process.env["npm_config_userconfig"] || path3.join(os.homedir(), ".npmrc");
  const npmConfigContents = await readConfigFile(npmConfigFile, "npm", ini.parse);
  npmConfigContents["@ronin:registry"] = "https://ronin.supply";
  npmConfigContents["//ronin.supply/:_authToken"] = token;
  await writeConfigFile(npmConfigFile, ini.stringify(npmConfigContents));
};
var storeTokenForBun = async (token) => {
  const bunConfigFile = path3.join(os.homedir(), ".bunfig.toml");
  const bunConfigContents = await readConfigFile(bunConfigFile, "Bun", toml.parse);
  if (!bunConfigContents.install) bunConfigContents.install = {};
  if (!bunConfigContents.install.scopes) bunConfigContents.install.scopes = {};
  if (!bunConfigContents.install.scopes.ronin) bunConfigContents.install.scopes.ronin = {};
  bunConfigContents.install.scopes.ronin.url = "https://ronin.supply";
  bunConfigContents.install.scopes.ronin.token = token;
  await writeConfigFile(bunConfigFile, toml.stringify(bunConfigContents));
};

// src/bin/commands/login.ts
var login_default = async (appToken) => {
  const spinner = ora2("Logging in").start();
  if (appToken) {
    await Promise.all([storeTokenForNPM(appToken), storeTokenForBun(appToken)]);
    spinner.succeed("Logged in successfully!");
    return;
  }
  let sessionToken;
  const port = await getPort();
  const server = http.createServer().listen(port);
  const baseURL = new URL("https://ronin.co/actions/auth/cli");
  const currentHost = `http://localhost:${port}`;
  const initURL = new URL(baseURL);
  initURL.searchParams.set("from", currentHost);
  spinner.text = `Please log in using the following URL:
${initURL.href}`;
  spinner.suffixText = `
`;
  try {
    [sessionToken] = await Promise.all([
      new Promise((resolve, reject) => {
        server.once("request", (req, res) => {
          res.setHeader("connection", "close");
          const requestURL = new URL(req.url || "", currentHost);
          const requestToken = requestURL.searchParams.get("token");
          if (!requestToken) return reject(new Error("Missing token"));
          resolve(requestToken);
          const headers = { Location: baseURL.href };
          res.writeHead(307, headers).end();
        });
        server.once("error", reject);
      }),
      open(initURL.href)
    ]);
  } finally {
    server.close();
  }
  await Promise.all([
    storeSession(sessionToken),
    storeTokenForNPM(sessionToken),
    storeTokenForBun(sessionToken)
  ]);
  spinner.suffixText = "";
  spinner.succeed("Logged in successfully!");
};

// src/bin/commands/sync.ts
import { select } from "@inquirer/prompts";
import ora3 from "ora";
import path6 from "path";

// src/bin/utils/config.ts
import fs4 from "fs";
import path4 from "path";
var saveConfig = (config) => {
  const configDir = path4.join(process.cwd(), ".ronin");
  if (!fs4.existsSync(configDir)) {
    fs4.mkdirSync(configDir);
  }
  const configPath = path4.join(configDir, "config.json");
  let existingConfig = {};
  if (fs4.existsSync(configPath)) {
    existingConfig = JSON.parse(fs4.readFileSync(configPath, "utf-8"));
  }
  fs4.writeFileSync(configPath, JSON.stringify({ ...existingConfig, ...config }, null, 2));
};
var resetConfig = () => {
  const configPath = path4.join(process.cwd(), ".ronin", "config.json");
  if (fs4.existsSync(configPath)) {
    fs4.unlinkSync(configPath);
  }
};
var readConfig = () => {
  const configPath = path4.join(process.cwd(), ".ronin", "config.json");
  if (!fs4.existsSync(configPath)) {
    return {};
  }
  return JSON.parse(fs4.readFileSync(configPath, "utf-8"));
};

// src/bin/utils/schema.ts
import fs5 from "fs/promises";
import path5 from "path";
import pluralize from "pluralize";
import ts from "typescript";

// src/utils/id.ts
import { init } from "@paralleldrive/cuid2";
var generateUniqueId = (length = 24) => init({ length })();

// src/bin/utils/schema.ts
var FIELD_TYPES = [
  "ShortText",
  "LongText",
  "RichText",
  "Time",
  "Blob",
  "Toggle",
  "Number",
  "Token",
  "JSON"
];
var generateFieldId = (type) => {
  return `${type}-${generateUniqueId()}`;
};
function convertToReadableText(str) {
  if (!str || typeof str !== "string") return null;
  const spacedStr = str.replace(/([a-z])([A-Z])/g, "$1 $2");
  const words = spacedStr.split(" ");
  const capitalizedWords = words.map((word) => word.charAt(0).toUpperCase() + word.slice(1));
  return capitalizedWords.join(" ");
}
async function parseSchemaDefinitionFile(filePath = "./schemas/index.ts", onError) {
  const relativePath = path5.relative(process.cwd(), filePath);
  const fullPath = path5.resolve(process.cwd(), filePath);
  const schemaFileExists = await exists(filePath);
  if (!schemaFileExists) {
    throw new Error(`The given path to the schema definition file does not exist: ${fullPath}`);
  }
  const fileContent = await fs5.readFile(fullPath, "utf-8");
  const sourceFile = ts.createSourceFile("temp.ts", fileContent, ts.ScriptTarget.Latest, true);
  const results = [];
  const typeMapping = {};
  const namespaceMapping = {};
  const schemaProperties = {};
  const missingSchemas = [];
  const unknownFields = [];
  let schemaRecordAlias = "SchemaRecord";
  let schemaRecordsAlias = "SchemaRecords";
  function getFieldType(type) {
    const originalType = typeMapping[type] || type;
    switch (originalType) {
      case "string":
      case "ShortText":
        return "short-text";
      case "LongText":
        return "long-text";
      case "RichText":
        return "rich-text";
      case "Date":
      case "Time":
        return "time";
      case "Blob":
        return "blob";
      case "boolean":
      case "Toggle":
        return "toggle";
      case "number":
      case "Number":
        return "number";
      case "Token":
        return "token";
      case "JSON":
        return "json";
      default: {
        const { resolvedTypeName } = resolveTypeAlias(originalType);
        if (resolvedTypeName === schemaRecordAlias) {
          return "record";
        }
        return "unknown";
      }
    }
  }
  function parseJsDoc(node) {
    let name = "";
    let description = "";
    const details = {};
    const commentsAndTags = ts.getJSDocCommentsAndTags(node);
    if (commentsAndTags.length > 0) {
      const jsDoc = commentsAndTags[0];
      const commentText = jsDoc.comment ? ts.getTextOfJSDocComment(jsDoc.comment) : void 0;
      if (commentText) {
        const lines = commentText.split("\n").map((line) => line.trim()).filter((line) => line);
        if (lines.length > 0) {
          name = lines[0];
          description = lines.slice(1).join(" ");
        }
      }
      if ("tags" in jsDoc && jsDoc.tags) {
        jsDoc.tags.forEach((tag) => {
          const tagName = tag.tagName.getText();
          const tagComment = ts.getTextOfJSDocComment(tag.comment);
          if (tagComment) {
            details[tagName] = tagComment;
          }
        });
      }
    }
    return { name, description, details };
  }
  function visitImports(node) {
    if (ts.isImportDeclaration(node) && node.importClause && node.importClause.namedBindings) {
      const namedBindings = node.importClause.namedBindings;
      if (ts.isNamedImports(namedBindings)) {
        namedBindings.elements.forEach((element) => {
          const importedName = element.name.text;
          const originalName = element.propertyName ? element.propertyName.text : importedName;
          typeMapping[importedName] = originalName;
          if (originalName === "SchemaRecord") {
            schemaRecordAlias = importedName;
          } else if (originalName === "SchemaRecords") {
            schemaRecordsAlias = importedName;
          }
        });
      } else if (ts.isNamespaceImport(namedBindings)) {
        namespaceMapping[namedBindings.name.text] = node.moduleSpecifier.getText().replace(/['"]/g, "");
      }
    }
  }
  function visitModule(node) {
    if (ts.isModuleDeclaration(node) && node.name.text === "ronin") {
      if (node.body && ts.isModuleBlock(node.body)) {
        node.body.statements.forEach((statement) => {
          if (ts.isInterfaceDeclaration(statement) && statement.name.text === "Schemas") {
            visitInterface(statement);
          }
        });
      }
    }
  }
  function visitInterface(node) {
    node.members.forEach((member) => {
      if (ts.isPropertySignature(member)) {
        const propertyName = member.name.getText();
        const typeName = member.type && ts.isTypeReferenceNode(member.type) ? member.type.typeName.getText() : "";
        if (typeName) {
          schemaProperties[propertyName] = typeName;
        }
      }
    });
    Object.keys(schemaProperties).forEach((propertyName) => {
      const typeName = schemaProperties[propertyName];
      const parsedType = parseTypeAlias(typeName, propertyName);
      if (parsedType) {
        results.push(parsedType);
      }
    });
  }
  function resolveTypeAlias(typeName) {
    let resolvedTypeName = typeName;
    let typeArguments = [];
    let namespace = null;
    function typeAliasVisitor(node) {
      if (ts.isTypeAliasDeclaration(node) && node.name.text === typeName) {
        if (ts.isTypeReferenceNode(node.type) && node.type.typeName) {
          const fullTypeName = node.type.typeName.getText();
          const parts = fullTypeName.split(".");
          if (parts.length > 1) {
            namespace = parts.slice(0, -1).join(".");
            resolvedTypeName = parts[parts.length - 1];
          } else {
            resolvedTypeName = fullTypeName;
          }
          if (node.type.typeArguments) {
            typeArguments = Array.from(node.type.typeArguments);
          }
        }
      }
      ts.forEachChild(node, typeAliasVisitor);
    }
    ts.forEachChild(sourceFile, typeAliasVisitor);
    return { resolvedTypeName, typeArguments, namespace };
  }
  function getLineAndColumnsNumber(node) {
    const start = node.getStart();
    const { line: startLine, character: startCharacter } = sourceFile.getLineAndCharacterOfPosition(start);
    const end = node.getEnd();
    const { line: endLine, character: endCharacter } = sourceFile.getLineAndCharacterOfPosition(end);
    return {
      start: { line: startLine + 1, character: startCharacter + 1 },
      end: { line: endLine + 1, character: endCharacter + 1 }
    };
  }
  function checkSchemaInclusion(node, parent) {
    const typeName = node.type.getText();
    const schemaProperty = Object.keys(schemaProperties).find((key) => schemaProperties[key] === typeName);
    if (schemaProperty) {
      return schemaProperty;
    } else {
      const source = getLineAndColumnsNumber(node);
      missingSchemas.push({
        name: typeName,
        parent,
        source: `${relativePath}:${source.start.line}:${source.start.character}`
      });
      return null;
    }
  }
  function getTypeFromNamespace(typeName) {
    const [namespace, type] = typeName.split(".");
    if (namespaceMapping[namespace]) {
      return typeMapping[type] || type;
    }
    return typeName;
  }
  function parseTypeAlias(typeName, propertyName) {
    const result = {
      name: convertToReadableText(typeName),
      slug: propertyName,
      pluralName: "",
      pluralSlug: "",
      fields: []
    };
    Object.keys(schemaProperties).forEach((schemaPropertyName) => {
      const schemaPropertyType = schemaProperties[schemaPropertyName];
      const { resolvedTypeName, typeArguments } = resolveTypeAlias(schemaPropertyType);
      const firstTypeArgument = typeArguments[0]?.getText();
      if (resolvedTypeName === schemaRecordsAlias && typeName === firstTypeArgument) {
        result.pluralName = convertToReadableText(schemaPropertyType);
        result.pluralSlug = schemaPropertyName;
      }
    });
    function typeAliasVisitor(node) {
      if (ts.isTypeAliasDeclaration(node) && node.name.text === typeName) {
        if (ts.isTypeReferenceNode(node.type) && node.type.typeArguments) {
          const typeLiteral = node.type.typeArguments[0];
          if (ts.isTypeLiteralNode(typeLiteral)) {
            typeLiteral.members.forEach((member) => {
              if (ts.isPropertySignature(member)) {
                const fieldName = member.name.getText();
                const memberTypeText = getTypeFromNamespace(member.type.getText());
                const fieldType = getFieldType(memberTypeText);
                const { name, description, details } = parseJsDoc(member);
                let schema = null;
                if (fieldType === "record") {
                  schema = checkSchemaInclusion(member, typeName);
                }
                if (fieldType === "unknown") {
                  const source = getLineAndColumnsNumber(member);
                  unknownFields.push({
                    parent: typeName,
                    name: fieldName,
                    type: memberTypeText,
                    source: `${relativePath}:${source.start.line}:${source.start.character}`
                  });
                }
                const field = {
                  id: generateFieldId(fieldType),
                  type: fieldType,
                  slug: fieldName,
                  name: name || fieldName.charAt(0).toUpperCase() + fieldName.slice(1),
                  unique: false,
                  required: !member.questionToken
                };
                if (description) {
                  field.description = description;
                }
                if (Object.keys(details).length > 0) {
                  field.details = details;
                }
                if (schema) {
                  field.schema = schema;
                }
                result.fields.push(field);
              }
            });
          }
        }
      }
      ts.forEachChild(node, typeAliasVisitor);
    }
    ts.forEachChild(sourceFile, typeAliasVisitor);
    if (!result.pluralName && result.fields.length) {
      const pluralTypeName = pluralize(typeName);
      onError(
        `The schema \`${typeName}\` does not have a plural slug and name defined.

Please define them in your schema definition file and include them in the \`Schemas\` interface:

type ${pluralTypeName} = ${schemaRecordsAlias}<${typeName}>;

interface Schemas {
  ${propertyName}: ${typeName};
  ${pluralize(result.slug)}: ${pluralTypeName};
}`
      );
    }
    return result.fields.length ? result : null;
  }
  function visit(node) {
    visitImports(node);
    visitModule(node);
    ts.forEachChild(node, visit);
  }
  visit(sourceFile);
  if (missingSchemas.length > 0) {
    const errorMessage = `The following schemas were used as a reference but weren't included in the \`Schemas\` interface:

${missingSchemas.map(({ name, parent, source }) => `  - \`${name}\` in \`${parent}\` (${source})`).join("\n")}

Please include them in the \`Schemas\` interface or remove their references.`;
    onError(errorMessage);
  }
  if (unknownFields.length > 0) {
    const errorMessage = "The type of the following fields could not be determined:\n\n" + Array.from(unknownFields).map(
      ({ name, parent, type, source }) => `  - \`${parent}.${name}\` is typed as \`${type}\` (${source})`
    ).join("\n") + "\n\nPlease make sure that the field is typed as any of the available field types exported from the `ronin/schema` module:\n\n" + FIELD_TYPES.map((type) => `  - ${type}`).join("\n") + "\n  - or a reference to another schema.";
    onError(errorMessage);
  }
  return results;
}

// src/bin/utils/sync.ts
import { confirm } from "@inquirer/prompts";
var getSpaces = async (sessionToken) => {
  let text;
  try {
    const res = await fetch("https://ronin.supply/-/ronin/spaces", {
      method: "GET",
      headers: {
        "Content-Type": "application/json",
        Authorization: `Bearer ${sessionToken}`
      }
    });
    text = await res.text();
    if (!res.ok) {
      throw new Error(text);
    }
  } catch (err) {
    throw new Error(`Failed to fetch available spaces: ${err.message}`);
  }
  let json;
  try {
    json = JSON.parse(text);
  } catch (err) {
    throw new Error(`Failed to parse response from server: ${text}`);
  }
  return json.spaces;
};
var getSchemas = async (token, space) => {
  let text;
  try {
    const res = await fetch(`https://ronin.supply/-/ronin/schemas?data-selector=${space}`, {
      method: "GET",
      headers: {
        "Content-Type": "application/json",
        Authorization: `Bearer ${token}`
      }
    });
    text = await res.text();
    if (!res.ok) {
      throw new Error(text);
    }
  } catch (err) {
    throw new Error(`Failed to fetch remote schemas: ${err.message}`);
  }
  let json;
  try {
    json = JSON.parse(text);
  } catch (err) {
    throw new Error(`Failed to parse response from server: ${text}`);
  }
  return json.schemas;
};
var replaceFieldIdsWithExisting = async (local, remote) => {
  return local.map((schema) => {
    const existingSchema = remote.find((s) => s.slug === schema.slug);
    if (!existingSchema) {
      return schema;
    }
    const existingFields = existingSchema.fields || [];
    const localFields = schema.fields || [];
    const fields = localFields.map((field) => {
      const existingField = existingFields.find((f) => f.slug === field.slug);
      if (!existingField) {
        return field;
      }
      return {
        ...field,
        id: existingField.id
      };
    });
    return {
      ...schema,
      fields
    };
  });
};
var compareSchemas = async (local, remote, spinner) => {
  const fieldsWithDifferentTypes = [];
  local.forEach(({ slug: schemaSlug, fields }) => {
    if (!fields) return;
    const existingSchema = remote.find((schema) => schema.slug === schemaSlug);
    const existingFields = existingSchema?.fields || [];
    if (!existingSchema) {
      return;
    }
    fields.forEach((field) => {
      const existingField = existingFields.find((f) => f.slug === field.slug);
      if (!existingField) {
        return;
      }
      const canChangeType = ["short-text", "long-text"].includes(existingField.type) && ["short-text", "long-text"].includes(field.type);
      if (!canChangeType && existingField.type !== field.type) {
        field.id = generateFieldId(field.type);
        fieldsWithDifferentTypes.push(`${schemaSlug}.${field.slug} (${existingField.type} -> ${field.type})`);
      }
    });
  });
  if (fieldsWithDifferentTypes.length === 0) {
    return;
  }
  spinner.warn(
    `The data type of the following fields have changed:
${fieldsWithDifferentTypes.map((field) => `  - ${field}`).join("\n")}

`
  );
  const answer = await confirm({
    message: "The content of these fields will be lost. Do you want to continue?",
    default: false
  });
  if (!answer) {
    spinner.stop();
    process.exit();
  }
};

// src/bin/commands/sync.ts
var DEFAULT_SCHEMA_SUMMARY = "This is a newly created schema.";
var sync_default = async (positionals2, appToken, sessionToken) => {
  if (positionals2.includes("-r") || positionals2.includes("--reset")) {
    resetConfig();
  }
  let status = "readingConfig";
  const spinner = ora3("Reading configuration").start();
  const token = appToken || sessionToken;
  if (!token) {
    throw new Error("Either an app token or session token is required to sync schemas.");
  }
  const customDir = positionals2[0] && !positionals2[0].startsWith("-") ? positionals2[0] : void 0;
  if (customDir) {
    const schemaFile = path6.join(customDir, "index.ts");
    const schemaFileExists = await exists(schemaFile);
    if (!schemaFileExists) {
      spinner.fail(
        "The provided schema directory does not exist or does not contain a schema definitions file (`index.ts`)."
      );
      process.exit(1);
    }
    saveConfig({ schemasDir: customDir });
  }
  const config = readConfig();
  try {
    if (!config.spaceId && !appToken && sessionToken) {
      const spaces = await getSpaces(sessionToken);
      if (!spaces || spaces.length === 0) {
        spinner.fail(
          "You don't have access to any space or your CLI session is invalid.\n\nPlease login again (by running `npx ronin login`) or create a new space on the dashboard (`https://ronin.co/new`) and try again."
        );
        process.exit(1);
      }
      if (spaces.length === 1) {
        config.spaceId = spaces[0].id;
        saveConfig({ spaceId: spaces[0].id });
      } else {
        spinner.stop();
        const answer = await select({
          message: "Which space do you want to sync schemas to?",
          choices: spaces.map((space) => ({
            name: space.handle,
            value: space.id,
            description: space.name
          }))
        });
        config.spaceId = answer;
        saveConfig({ spaceId: answer });
      }
    }
    status = "readingSchemas";
    spinner.text = "Reading schema definitions";
    const schemaFile = path6.join(config.schemasDir || "schemas", "index.ts");
    let schemaDefinitions = await parseSchemaDefinitionFile(schemaFile, (error) => {
      spinner.fail(error);
      process.exit(1);
    });
    status = "comparing";
    spinner.start("Retrieving existing schemas");
    const remoteSchemas = await getSchemas(token, config.spaceId);
    schemaDefinitions = await replaceFieldIdsWithExisting(schemaDefinitions, remoteSchemas);
    schemaDefinitions = schemaDefinitions.map((schema) => ({
      ...schema,
      summary: schema.summary || remoteSchemas.find((s) => s.id === schema.id)?.summary || DEFAULT_SCHEMA_SUMMARY
    }));
    status = "comparing";
    spinner.text = "Comparing local and remote schema definitions";
    await compareSchemas(schemaDefinitions, remoteSchemas, spinner);
    status = "syncing";
    spinner.text = "Syncing schema definitions";
    const res = await fetch("https://ronin.supply/-/ronin/sync", {
      method: "POST",
      headers: {
        "Content-Type": "application/json",
        Authorization: `Bearer ${appToken || sessionToken}`
      },
      body: JSON.stringify({ schemas: schemaDefinitions, space: config.spaceId })
    });
    const text = await res.text();
    if (!res.ok) {
      throw new Error(text);
    }
  } catch (err) {
    spinner.fail(
      `Failed to ${status === "readingSchemas" ? "read schema definitions" : "apply new schema changes"}:
`
    );
    console.error(err);
    process.exit(1);
  }
  spinner.succeed("Schemas are in sync!");
};

// src/bin/utils/info.ts
import chalkTemplate from "chalk-template";

// package.json
var version = "4.1.3";

// src/bin/utils/info.ts
var printVersion = () => {
  console.log(version);
  process.exit(0);
};
var printHelp = () => {
  const text = chalkTemplate`
  {bold.magenta ronin} — Data at the edge

  {bold USAGE}

      {bold $} {bold.magenta ronin}
      {bold $} {bold.magenta ronin} login
      {bold $} {bold.magenta ronin} --help
      {bold $} {bold.magenta ronin} --version

  {bold COMMANDS}

      login                               Authenticate with RONIN (run by default for every command)
      init [space]                        Initialize the TypeScript types for a given space
      sync [dir]                          Read local schema definitions (from a given directory) and sync them with RONIN

  {bold OPTIONS}

      -h, --help                          Shows this help message
      -v, --version                       Shows the version of the CLI that is currently installed
      -d, --debug                         Shows additional debugging information
  `;
  console.log(text);
  process.exit(0);
};

// src/bin/index.ts
var values;
var positionals;
try {
  ({ values, positionals } = parseArgs({
    args: process.argv,
    options: {
      help: {
        type: "boolean",
        short: "h",
        default: false
      },
      version: {
        type: "boolean",
        short: "v",
        default: false
      },
      debug: {
        type: "boolean",
        short: "d",
        default: false
      }
    },
    strict: true,
    allowPositionals: true
  }));
} catch (err) {
  if (err instanceof Error) {
    console.error(err.message);
  } else {
    throw err;
  }
  process.exit(1);
}
var run = async () => {
  if (values.help) return printHelp();
  if (values.version) return printVersion();
  const normalizedPositionals = positionals.map((positional) => positional.toLowerCase());
  const appToken = process.env.RONIN_TOKEN;
  const session = await getSession();
  if (!process.stdout.isTTY && !session && !appToken) {
    let message = "If RONIN CLI is invoked from a non-interactive shell, ";
    message += "a `RONIN_TOKEN` environment variable containing an app token must be provided.";
    console.error(message);
    process.exit(1);
  }
  if (!session && !normalizedPositionals.includes("login")) await login_default(appToken);
  if (normalizedPositionals.includes("login")) return login_default(appToken);
  if (normalizedPositionals.includes("init")) return init_default(positionals);
  if (normalizedPositionals.includes("sync")) {
    return sync_default(positionals.slice(positionals.indexOf("sync") + 1), appToken, session?.token);
  }
  return printHelp();
};
run();
process.on("SIGINT", () => {
  process.exit(0);
});
process.on("SIGTERM", () => {
  process.exit(0);
});
